@@ -126,36 +126,36 @@ which details how to sub-class the distribution.
 use 5.006;
 use strict;
 use warnings;
-use Carp                       ();
-use Archive::Tar               ();
-use Archive::Zip               ();
-use File::Spec                 ();
-use File::Spec::Unix           ();
-use File::Spec::Win32          ();
-use File::Copy                 ();
-use File::Copy::Recursive      ();
-use File::Path                 ();
-use File::PathList             ();
-use File::pushd                ();
-use File::Remove               ();
-use File::Basename             ();
-use IPC::Run3                  ();
-use Params::Util               ();
-use HTTP::Status               ();
-use LWP::UserAgent             ();
-use LWP::UserAgent::Determined ();
-use LWP::Online                ();
-use Module::CoreList           ();
-use Template                   ();
-use PAR::Dist                  ();
-use Portable::Dist             ();
-use Storable                   ();
-use URI::file                  ();
-use Perl::Dist::Inno::Script   ();
+use Carp                      ();
+use Archive::Tar              ();
+use Archive::Zip              ();
+use File::Spec                ();
+use File::Spec::Unix          ();
+use File::Spec::Win32         ();
+use File::Copy                ();
+use File::Copy::Recursive     ();
+use File::Path                ();
+use File::PathList            ();
+use File::pushd               ();
+use File::Remove              ();
+use File::Basename            ();
+use File::HomeDir             ();
+use IPC::Run3                 ();
+use Params::Util              ();
+use HTTP::Status              ();
+use LWP::UserAgent            ();
+use LWP::Online               ();
+use Module::CoreList          ();
+use Template                  ();
+use PAR::Dist                 ();
+use Portable::Dist            ();
+use Storable                  ();
+use URI::file                 ();
+use Perl::Dist::Inno::Script  ();
 
 use vars qw{$VERSION @ISA};
 BEGIN {
-        $VERSION  = '1.10';
+        $VERSION  = '1.11';
 	@ISA      = 'Perl::Dist::Inno::Script';
 }
 
@@ -175,14 +175,11 @@ use Object::Tiny qw{
 	build_dir
 	checkpoint_dir
 	iss_file
-	user_agent
 	bin_perl
 	bin_make
 	bin_pexports
 	bin_dlltool
 	env_path
-	env_lib
-	env_include
 	debug_stdout
 	debug_stderr
 	output_file
@@ -428,12 +425,6 @@ sub new {
 	}
 
 	# Auto-detect online-ness if needed
-	unless ( defined $self->user_agent ) {
-		$self->{user_agent} = LWP::UserAgent->new(
-			agent   => "$class/" . $VERSION || '0.00',
-			timeout => 30,
-		);
-	}
 	unless ( defined $self->offline ) {
 		$self->{offline} = LWP::Online::offline();
 	}
@@ -543,8 +534,6 @@ sub new {
 
 	# Inno-Setup Initialization
 	$self->{env_path}    = [];
-	$self->{env_lib}     = [];
-	$self->{env_include} = [];
 	$self->add_dir('c');
 	$self->add_dir('perl');
 	$self->add_dir('licenses');
@@ -659,12 +648,46 @@ sub checkpoint_save {
 	}
 
 	# Store the main object.
-	# Blank the checkpoint values to prevent load/save loops
-	Storable::nstore( {
+	# Blank the checkpoint values to prevent load/save loops, and remove
+	# things we can recreate later.
+	my $copy = {
 		%$self,
 		checkpoint_before => 0,
 		checkpoint_after  => 0,
-	}, $self->checkpoint_file );
+		user_agent        => undef,
+	};
+	Storable::nstore( $copy, $self->checkpoint_file );
+
+	return 1;
+}
+
+sub checkpoint_load {
+	my $self = shift;
+	unless ( $self->temp_dir ) {
+		die "Checkpoints require a temp_dir to be set";
+	}
+
+	# Does the checkpoint exist
+	$self->trace("Removing old checkpoint\n");
+	$self->{checkpoint_dir} = File::Spec->catfile(
+		$self->temp_dir, 'checkpoint',
+	);
+	unless ( -d $self->checkpoint_dir ) {
+		die "Failed to find checkpoint directory";
+	}
+
+	# Load the stored hash over our object
+	my $stored = Storable::retrieve( $self->checkpoint_file );
+	%$self = %$stored;
+
+	# Pull all the directories out of the storage
+	$self->trace("Restoring checkpoint directories...\n");
+	foreach my $dir ( qw{ build_dir download_dir image_dir output_dir } ) {
+		my $from = File::Spec->catdir( $self->checkpoint_dir, $dir );
+		my $to   = $self->$dir();
+		File::Remove::remove( $to );
+		$self->_copy( $from => $to );
+	}
 
 	return 1;
 }
@@ -786,6 +809,9 @@ sub run {
 		return 1;
 	}
 
+	# Don't buffer
+	$| = 1;
+
 	# Install the core C toolchain
 	$self->checkpoint_task( install_c_toolchain  => 1 );
 
@@ -966,8 +992,6 @@ sub install_cpan_upgrades {
 	}
 
 	# Generate the CPAN installation script
-	# my $env_lib     = $self->get_env_lib;
-	# my $env_include = $self->get_env_include;
 	my $cpan_string = <<"END_PERL";
 print "Loading CPAN...\\n";
 use CPAN;
@@ -1214,9 +1238,7 @@ sub install_perl_588_bin {
 
 	# Build win32 perl
 	SCOPE: {
-		my $wd = File::pushd::pushd(
-			File::Spec->catdir( $unpack_to, $perlsrc , "win32" ),
-		);
+		my $wd = $self->_pushd($unpack_to, $perlsrc , "win32" );
 
 		# Prepare to patch
 		my $image_dir  = $self->image_dir;
@@ -1279,7 +1301,7 @@ sub install_perl_589 {
 	# Install the main perl distributions
 	$self->install_perl_589_bin(
 		name       => 'perl',
-		url        => 'http://strawberryperl.com/package/perl-5.8.9-RC1.tar.gz',
+		url        => 'http://strawberryperl.com/package/perl-5.8.9.tar.gz',
 		unpack_to  => 'perl',
 		install_to => 'perl',
 		patch      => [ qw{
@@ -1344,9 +1366,7 @@ sub install_perl_589_bin {
 
 	# Build win32 perl
 	SCOPE: {
-		my $wd = File::pushd::pushd(
-			File::Spec->catdir( $unpack_to, $perlsrc , "win32" ),
-		);
+		my $wd = $self->_pushd($unpack_to, $perlsrc , "win32" );
 
 		# Prepare to patch
 		my $image_dir  = $self->image_dir;
@@ -1527,9 +1547,7 @@ sub install_perl_5100_bin {
 
 	# Build win32 perl
 	SCOPE: {
-		my $wd = File::pushd::pushd(
-			File::Spec->catdir( $unpack_to, $perlsrc , "win32" ),
-		);
+		my $wd = $self->_pushd($unpack_to, $perlsrc , "win32" );
 
 		# Prepare to patch
 		my $image_dir  = $self->image_dir;
@@ -2205,7 +2223,7 @@ sub install_distribution {
 
 	# Build the module
 	SCOPE: {
-		my $wd = File::pushd::pushd( $unpack_to );
+		my $wd = $self->_pushd($unpack_to);
 
 		# Enable automated_testing mode if needed
 		# Blame Term::ReadLine::Perl for needing this ugly hack.
@@ -2278,8 +2296,6 @@ sub install_module {
 	}
 
 	# Generate the CPAN installation script
-	# my $env_lib     = $self->get_env_lib;
-	# my $env_include = $self->get_env_include;
 	my $cpan_string = <<"END_PERL";
 print "Loading CPAN...\\n";
 use CPAN;
@@ -2654,24 +2670,6 @@ sub write_exe {
 		}
 		$self->add_env( PATH => $value );
 	}
-	if ( @{$self->{env_lib}} ) {
-		my $value = "{olddata}";
-		foreach my $array ( @{$self->{env_lib}} ) {
-			$value .= File::Spec::Win32->catdir(
-				';{app}', @$array,
-			);
-		}
-		$self->add_env( LIB => $value );
-	}
-	if ( @{$self->{env_include}} ) {
-		my $value = "{olddata}";
-		foreach my $array ( @{$self->{env_include}} ) {
-			$value .= File::Spec::Win32->catdir(
-				';{app}', @$array,
-			);
-		}
-		$self->add_env( INCLUDE => $value );
-	}
 
 	$self->SUPER::write_exe(@_);
 }
@@ -2955,6 +2953,57 @@ sub file {
 	File::Spec->catfile( shift->image_dir, @_ );
 }
 
+sub user_agent {
+	my $self = shift;
+	unless ( $self->{user_agent} ) {
+		if ( $self->{user_agent_cache} ) {
+			SCOPE: {
+				# Temporarily set $ENV{HOME} to the File::HomeDir
+				# version while loading the module.
+				local $ENV{HOME} ||= File::HomeDir->my_home;
+				require LWP::UserAgent::WithCache;
+			}
+			$self->{user_agent} = LWP::UserAgent::WithCache->new( {
+				namespace          => 'perl-dist',
+				cache_root         => $self->user_agent_directory,
+				cache_depth        => 0,
+				default_expires_in => 86400 * 30,
+				show_progress      => 1,
+			} );
+		} else {
+			$self->{user_agent} = LWP::UserAgent->new(
+				agent         => ref($self) . '/' . ($VERSION || '0.00'),
+				timeout       => 30,
+				show_progress => 1,
+			);
+		}
+	}
+	return $self->{user_agent};
+}
+
+sub user_agent_cache {
+	$_[0]->{user_agent_cache};
+}
+
+sub user_agent_directory {
+	my $self = shift;
+	my $path = ref($self);
+	   $path =~ s/::/-/g;
+	my $dir  = File::Spec->catdir(
+		File::HomeDir->my_data,
+		'Perl', $path,
+	);
+	unless ( -d $dir ) {
+		unless ( File::Path::mkpath( $dir, { verbose => 0 } ) ) {
+			die("Failed to create $dir");
+		}
+	}
+	unless ( -w $dir ) {
+		die("No write permissions for LWP::UserAgent cache '$dir'");
+	}
+	return $dir;
+}
+
 sub _mirror {
 	my ($self, $url, $dir) = @_;
 	my $file = $url;
@@ -2972,7 +3021,7 @@ sub _mirror {
 
 	$self->trace("Downloading file $url...\n");
 	if ( $url =~ m|^file://| ) {
-		# Don't use Determined for files (it generates warnings)
+		# Don't use WithCache for files (it generates warnings)
 		my $ua = LWP::UserAgent->new;
 		my $r  = $ua->mirror( $url, $target );
 		if ( $r->is_error ) {
@@ -2981,7 +3030,8 @@ sub _mirror {
 			$self->trace("(already up to date)\n");
 		}
 	} else {
-		my $ua = LWP::UserAgent::Determined->new;
+		# my $ua = $self->user_agent;
+		my $ua = LWP::UserAgent->new;
 		my $r  = $ua->mirror( $url, $target );
 		if ( $r->is_error ) {
 			$self->trace("    Error getting $url:\n" . $r->as_string . "\n");
@@ -3025,6 +3075,13 @@ sub _move {
 	File::Copy::Recursive::rmove( $from, $to ) or die $!;
 }
 
+sub _pushd {
+	my $self = shift;
+	my $dir  = File::Spec->catdir(@_);
+	$self->trace("Lexically changing directory to $dir...\n");
+	return File::pushd::pushd( $dir );
+}
+
 sub _make {
 	my $self   = shift;
 	my @params = @_;
@@ -3080,8 +3137,7 @@ sub _run3 {
 sub _extract {
 	my ( $self, $from, $to ) = @_;
 	File::Path::mkpath($to);
-	my $wd = File::pushd::pushd( $to );
-	$|++;
+	my $wd = $self->_pushd($to);
 	$self->trace("Extracting $from...\n");
 	if ( $from =~ m{\.zip\z} ) {
 		my $zip = Archive::Zip->new( $from );
@@ -3103,7 +3159,7 @@ sub _extract_filemap {
 
 	if ( $archive =~ m{\.zip\z} ) {
 		my $zip = Archive::Zip->new( $archive );
-		my $wd = File::pushd::pushd( $basedir );
+		my $wd  = $self->_pushd($basedir);
 		while ( my ($f, $t) = each %$filemap ) {
 			$self->trace("Extracting $f to $t\n");
 			my $dest = File::Spec->catfile( $basedir, $t );
@@ -3262,7 +3318,7 @@ L<Perl::Dist>, L<vanillaperl.com>, L<http://ali.as/>
 
 =head1 COPYRIGHT
 
-Copyright 2008 Adam Kennedy.
+Copyright 2009 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
